# IIDR-Masker (simple-masker)
Демонстрация базовых возможностей маскирования/хеширования и преобразований 
типов данных для IBM Change Data Capture.

Предварительно скомпилированные классы, готовые к установке, 
доступны в каталоге `bin`. Для использования эти классы необходимо
разместить в каталоге `{cdc-install-dir}/lib` агента IBM CDC.

При работе с порождёнными колонками (derived columns)
установка выполняется на стороне исходного агента.
В случае непосредственного сопоставления с целевыми колонками
порождённых выражений (derived expressions) установка выполняется
на стороне агента-получателя.

Для сборки проекта с помощью Maven необходимо извлечь файл `lib/ts.jar`
из вашей инсталляции агента IBM CDC. Библиотеку `ts.jar` необходимо
поместить в локальный репозиторий Maven:

```bash
mvn install:install-file -Dfile=`pwd`/ts.jar \
  -DgroupId=com.ibm.iidr -DartifactId=ts -Dversion=11.4.0.4.5607 -Dpackaging=jar
```

После этого сборку и открытие проекта в различных Java IDE можно 
осуществлять с помощью файла `pom.xml`.

Альтернативный метод сборки заключается в копировании файлов `*.java` 
в каталог `{cdc-install-dir}/lib` и последующей компиляции 
с помощью команды `javac`:

```bash
javac CdcHasher.java -classpath ts.jar
javac CdcBin2Str.java -classpath ts.jar
```

## Использование CdcHasher

Расчёт хэш-кода над колонкой может настраиваться в подписках 
IBM CDC следующим синтаксисом:

```
%USERFUNC("JAVA","CdcHasher", "SHA-1", PAN)
%USERFUNC("JAVA","CdcHasher", "SHA-1", PAN, 0)
%USERFUNC("JAVA","CdcHasher", "SHA-1", PAN, 1)
```

Здесь `PAN` - имя исходной колонки для расчёта хэша, `"SHA-1"` - тип хэша.
Опциональный третий аргумент, если его установить в ненулевое значение,
позволяет вернуть хеш в формате Base64, который даёт более компактные
значения по сравнению с используемым по умолчанию форматом Hex.

Состав поддерживаемых алгоритмов хэширования приведён по ссылке ниже,
см. тип алгоритмов "MessageDigest":
https://www.ibm.com/support/knowledgecenter/en/SSYKE2_8.0.0/com.ibm.java.security.component.80.doc/security-component/pkcs11implDocs/supportedalgorithms.html

## Использование CdcBin2Str

`CdcBin2Str` обеспечивает преобразование двоичных (или даже текстовых) значений
в шестнадцатиричное или Base64 текстовое представление, с использованием
следующего синтаксиса:

```
%USERFUNC("JAVA","CdcBin2Str", uuid)        -- hex conversion
%USERFUNC("JAVA","CdcBin2Str", uuid, 0)     -- hex conversion
%USERFUNC("JAVA","CdcBin2Str", uuid, 1)     -- base64 conversion
```

Недвоичные данные (текст, числа, даты и т.п.) сперва преобразуются в текст 
(операция пропускается для собственно текста), затем этот текст переводится 
в двоичный формат с использованием кодировки UTF-8, а затем уже двоичные 
данные переводятся в шестнадцатиричный или Base64 формат.
